unit DrehElement;
(***** BEGIN LICENSE BLOCK *****
Version: MPL 1.1

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is DrehElement.pas .

The Initial Developer of the Original Code is
Florian Kberle.
Portions created by the Initial Developer are Copyright (C) 2004
the Initial Developer. All Rights Reserved.

Contributor(s):

***** END LICENSE BLOCK *****)
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ZeichenRoutinen
  ;

type
  TDrehElement = class(TCustomControl)
  private
    FWinkel:Single;
    FMouseDown:Boolean;
    FButtonColor:TColor;
    FBorder:Integer;
    FSelColor:TColor;
    FOnChange:TNotifyEvent;
    function PunktzuWinkel(X,Y:Integer):Double;
    procedure SetButtonColor(const NewColor:TColor);
    procedure SetSelColor(const NewColor:TColor);
    procedure SetBorder(const NewBorder:Integer);
    procedure SetWinkel_Rad(const NeuerWinkel:Single);
    procedure SetWinkel_Grad(const NeuerWinkel:Integer);
    function  GetWinkel_Grad:Integer;
    function IsClicked(X,Y:Integer):Boolean;
    function Quadrat_fuer_Kreis(const Radius:Integer):TRect;
  protected
    procedure Paint;override;
    procedure DoExit;override;
    procedure DoEnter;override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
  public
    constructor Create(AOwner: TComponent); override;
  published
    property ButtonColor:TColor read FButtonColor write SetButtonColor;
    property SelColor:TColor read FSelColor write SetSelColor;
    property Border:Integer read FBorder write SetBorder;
    property Winkel_Grad:Integer read GetWinkel_Grad write SetWinkel_Grad;
    property Winkel_Rad:Single read FWinkel write SetWinkel_Rad;
    property OnChange:TNotifyEvent read FOnChange write FOnChange;
    property Color;
    property ParentColor;
    property ParentFont;
    property Font;
    property Align;
  end;

procedure Register;

implementation
function TDrehElement.Quadrat_fuer_Kreis(const Radius:Integer):TRect;
var
  Mitte:TPoint;
begin
  Mitte.x := ClientWidth div 2;
  Mitte.y := ClientHeight div 2;
  Result.Left   := Mitte.x - Radius;
  Result.Right  := Mitte.x + Radius;
  Result.Top    := Mitte.y - Radius;
  Result.Bottom := Mitte.y + Radius;
end;


constructor TDrehElement.Create(AOwner: TComponent);
begin
  inherited;
  DoubleBuffered := True;
  ControlStyle := ControlStyle+ [
                                csOpaque //Das Steuerelement fllt sein Client-Rechteck vollstndig aus.
                                ];
  FButtonColor := clSilver;
  FSelColor := clLime;
  FBorder := 10;
end;

procedure TDrehElement.SetBorder(const NewBorder:Integer);
begin
  if NewBorder <> FBorder then
  begin
    if NewBorder < 0 then FBorder := -NewBorder else FBorder := NewBorder;
    Repaint;
  end;
end;

procedure TDrehElement.SetButtonColor(const NewColor:TColor);
begin
  if NewColor <> FButtonColor then
  begin
    FButtonColor := NewColor;
    Repaint;
  end;
end;

procedure TDrehElement.SetSelColor(const NewColor:TColor);
begin
  if NewColor <> FSelColor then
  begin
    FSelColor := NewColor;
    if Focused or (csDesigning in ComponentState) then Repaint;
  end;
end;

procedure TDrehElement.DoExit;
begin
  inherited;
  Repaint;
end;
procedure TDrehElement.DoEnter;
begin
  inherited;
  Repaint;
end;



function TDrehElement.PunktzuWinkel(X,Y:Integer):Double;
begin
  X := X - (ClientWidth div 2);
  Y := Y - (ClientHeight div 2);
  if X = 0 then
   if Y > 0 then  result := 0.5*pi else result:= 1.5*pi
  else {X <> 0}
  begin
   if X > 0 then
   begin
     if Y < 0 then
      result := 2*pi+Arctan(Y/X)
     else
      result := Arctan(Y/X)
   end
   else
    result := Arctan(Y/X) +pi;
  end;
  Result := 2*pi - Result;
end;

function TDrehElement.IsClicked(X,Y:Integer):Boolean;
var
  Punkt:record X,Y:Single; end;
  Abstand,MaxRadius:Single;
begin
  Punkt.X := (X - ClientWidth/2);
  Punkt.Y := (Y - ClientHeight/2);
  Abstand:= Sqrt(Sqr(Punkt.X)+Sqr(Punkt.Y));
  if Width < Height then //Grt mglichen Radius ermitteln.
    MaxRadius := ClientWidth / 2
  else
    MaxRadius := ClientHeight / 2;
  Result := Abstand < 0.75*MaxRadius ;

end;
procedure TDrehElement.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited;
  if not IsClicked(X,Y) then Exit;

  if Button = mbLeft then
  begin
    FMouseDown:= True;
    if not Focused then SetFocus else Repaint;
  end;
end;

procedure TDrehElement.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  inherited;
  if FMouseDown and (Button = mbLeft) then
  begin
    FMouseDown:= false;
    Repaint;
  end;
end;

procedure TDrehElement.MouseMove(Shift: TShiftState; X, Y: Integer);
begin
  inherited;
  if FMouseDown then
  begin
    FWinkel := PunktzuWinkel(X,Y);
    if Assigned(FOnChange) then FOnChange(Self);
    Repaint;
  end;
end;

procedure TDrehElement.SetWinkel_Rad(const NeuerWinkel:Single);
begin
  if NeuerWinkel <> FWinkel then
  begin
    FWinkel := NeuerWinkel;
    Repaint;
  end;
end;
procedure TDrehElement.SetWinkel_Grad(const NeuerWinkel:Integer);
begin
 if GetWinkel_Grad <> NeuerWinkel then
 begin
   FWinkel := NeuerWinkel*pi/180;
   Repaint;
 end;
end;
function  TDrehElement.GetWinkel_Grad:Integer;
begin
  Result := round(FWinkel/pi*180)
end;
procedure TDrehElement.Paint;
const
  Sin45=0.70710678119; //sin(pi/4)
var
  MaxRadius:Integer;
  Radius:Integer;
  Zeiger:array [0..2] of TPoint;
  Bitmap:TBitmap;
  KreisBereich:TRect;
  Font_Size:Integer;
  Schrift_Position:TPoint;
  Text:String;
  Mitte:record X,Y:Single; end;
begin
  inherited;
  Mitte.x := Width div 2;
  Mitte.y := Height div 2;

  if Width < Height then //Grt mglichen Radius ermitteln.
    MaxRadius := ClientWidth div 2
  else
    MaxRadius := ClientHeight div 2;

  if MaxRadius <= 4 then //Das Steuerelement ist zu klein
  begin
    Canvas.Brush.Style := bsSolid;
    Canvas.Brush.Color := clBlack;
    Canvas.Pen.Style := psSolid;
    Canvas.Pen.Color := clBlack;
    Canvas.Ellipse(Quadrat_fuer_Kreis(MaxRadius));
    Exit;
  end
  else
   Radius := (MaxRadius*3) div 4;

  KreisBereich := Quadrat_fuer_Kreis(Radius);
  Bitmap := TBitmap.Create;
  try
    Bitmap.Width :=  ClientWidth;
    Bitmap.Height := ClientHeight;

    //Hintergrund fllen
    Bitmap.Canvas.Pen.Color := Color;
    Bitmap.Canvas.Brush.Color := Color;
    Bitmap.Canvas.Rectangle(ClientRect);

    //Falls ausgewhlt Ring zeichnen
    if Focused then
    begin
      DrawEllipse(Bitmap,Quadrat_fuer_Kreis((Radius + MaxRadius)div 2),SelColor,MaxRadius-Radius);
    end;

    //Zeiger berechnen;
    Zeiger[0].x := Round(Mitte.x+MaxRadius*cos(FWinkel));
    Zeiger[0].y := Round(Mitte.Y-MaxRadius*sin(FWinkel));
    Zeiger[1].X := Round(Mitte.x+Radius*cos(FWinkel+0.3));
    Zeiger[1].Y := Round(Mitte.y-Radius*sin(FWinkel+0.3));
    Zeiger[2].X := Round(Mitte.x+Radius*cos(FWinkel-0.3));
    Zeiger[2].Y := Round(Mitte.y-Radius*sin(FWinkel-0.3));

    //Zeiger zeichnen
    Bitmap.Canvas.Brush.Color := ButtonColor;
    Bitmap.Canvas.Pen.Color := ButtonColor;
    Bitmap.Canvas.Polygon(Zeiger);

    //Eigentlichen Button zeichnen
    DrawRoundButton(Bitmap,KreisBereich,ButtonColor,FBorder,FMouseDown);

    //Grenzen im Entwicklungs Modus zeichnen
    if csDesigning in ComponentState then
    begin
      Bitmap.Canvas.Pen.Style := psDot;
      Bitmap.Canvas.Pen.Color := SelColor;
      Bitmap.Canvas.Brush.Style := bsClear;
      Bitmap.Canvas.Ellipse(Quadrat_fuer_Kreis(MaxRadius));
      Bitmap.Canvas.Pen.Mode :=pmNot	;
      Bitmap.Canvas.Rectangle(ClientRect);
    end;

    Text := IntToStr(round(GetWinkel_Grad)) ;
    Font_Size := Round((Radius*Sin45)*(2/3));//Schriftdurchmesser durch 3(Zeichen);

    //Aussehen der Schrift festlegen
    Bitmap.Canvas.Font.Assign(Font);
    Bitmap.Canvas.Font.Size := Font_Size;
    Bitmap.Canvas.Brush.Style := bsClear;

    //Schrift Position ermitteln
    Schrift_Position.y := (ClientHeight-Bitmap.Canvas.TextHeight(Text)) div 2 ;
    Schrift_Position.x := (ClientWidth-Bitmap.Canvas.TextWidth(Text)) div 2 ;

    //Ab entsprechender Textgre Text ausgeben;
    if Font_Size >= 5 then
      Bitmap.Canvas.TextOut(Schrift_Position.x,Schrift_Position.y,Text);

    //Fertiges Bild zeichnen
    Canvas.Draw(0,0,Bitmap);
  finally
    Bitmap.Free;
  end;
end;

procedure Register;
begin
  RegisterComponents('Tutorial', [TDrehElement]);
end;

end.
