unit Graph;

interface
(***** BEGIN LICENSE BLOCK *****
Version: MPL 1.1

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is Graph.pas .

The Initial Developer of the Original Code is
Florian Kberle.
Portions created by the Initial Developer are Copyright (C) 2004
the Initial Developer. All Rights Reserved.

Contributor(s):

***** END LICENSE BLOCK *****)
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,imglist;

type
  TGraphFunction=function(X:Extended):Extended of object;

  TGraphColors=class(TPersistent)
  private
    FBackGround:TColor;
    FPen:TColor;
    OnChange:procedure of object;
    procedure SetBackGround(NewColor:TColor);
    procedure SetPen(NewColor:TColor);
  published
    property BackGround:TColor read FBackGround write SetBackGround;
    property Pen:TColor  read FPen  write SetPen;
  end;


  TGraph = class(TGraphicControl)
  private
    FGraphFunction:TGraphFunction;
    FMax,FMin:Real;
    FColors:TGraphColors;
    FTransparent:Boolean;
    procedure SetColors(NewColors:TGraphColors);
    procedure SetGraphFunction(NewFunction:TGraphFunction);
    procedure SetTransparent(NewValue:Boolean);
  protected
    procedure Paint; override;
    procedure SetName(const NewName: TComponentName);override;
  public
    Constructor Create(AOwner:TComponent);override;
    Destructor Destroy; override;
  published
    property Colors : TGraphColors read FColors write SetColors;
    property GraphFunction:TGraphFunction read FGraphFunction write SetGraphFunction;
    property Min:Real read FMin write FMin;
    property Max:Real read FMax write FMax;
    property Transparent: Boolean read FTransparent write SetTransparent;
    property Align;
  end;

procedure Register;

implementation



Constructor TGraph.Create(AOwner:TComponent);
begin
 inherited;
 FMax := 10;
 FMin := -5;
 FColors := TGraphColors.Create;
 FColors.OnChange := Repaint;//Bei nderung der Farbeinstellung wird die Komponente neu gezeichnet
 FColors.Pen := clblack;
 FColors.BackGround := clwhite;
end;
Destructor TGraph.Destroy;
begin
 inherited;
 FColors.free;
end;


procedure TGraph.Paint;
var
  I:Integer;
  X:Real;
  BufferBitmap:TBitmap;
  YFaktor:Real;
begin
 inherited;
 if csDesigning in ComponentState then
 begin
  canvas.pen.Color := Colors.Pen;
  if FTransparent then
   canvas.Brush.style := bsclear
  else
  begin
   canvas.Brush.style := bsSolid;
   canvas.Brush.color := Colors.BackGround;
  end;
  Canvas.Pen.style := psDashDot;
  Canvas.Rectangle(0,0,width,height);
  canvas.Font.Color := Colors.Pen;
  canvas.TextOut((width - Canvas.TextWidth(Name)) div 2,(height - Canvas.TextHeight(Name)) div 2,name);
  exit;
 end;
 if @FGraphFunction = nil then exit;

 if FTransparent then
 begin
   //StiftFarbe einstellen
  Canvas.Pen.Color := FColors.Pen;
  //Graph zeichnen
  YFaktor := width/(FMax-FMin);
  Canvas.MoveTo(0,height div 2 - Round( FGraphFunction(FMin)*YFaktor));
  for I := 0 to width do
  begin
   X := FMin + I/width*(FMax-FMin);
   Canvas.LineTo(I,height div 2 - Round( FGraphFunction(X)*YFaktor));
  end;
 end
 else//if not FTranspartent then
 begin
  BufferBitmap := TBitmap.Create;
  try
   //Richtiges Bitmap-Format einstellen
   BufferBitmap.Width  := width;
   BufferBitmap.Height := Height;
   BufferBitmap.PixelFormat := pf24bit;
   //Hintergrund fllen
   BufferBitmap.Canvas.Pen.Color := FColors.background;
   BufferBitmap.Canvas.brush.Color := FColors.background;
   BufferBitmap.Canvas.Rectangle(0,0,width,height);
   //StiftFarbe einstellen
   BufferBitmap.Canvas.Pen.Color := FColors.Pen;
   //Graph zeichnen
   YFaktor := width/(FMax-FMin);
   BufferBitmap.Canvas.MoveTo(0,height div 2 - Round( FGraphFunction(FMin)*YFaktor));
   for I := 0 to width do
   begin
    X := FMin + I/width*(FMax-FMin);
    BufferBitmap.Canvas.LineTo(I,height div 2 - Round( FGraphFunction(X)*YFaktor));
   end;
   Canvas.Draw(0,0,BufferBitmap);
  finally
   BufferBitmap.Free;
  end;
 end;
end;

{------------------------------------------------------------------------------}
//                            Procedure Set...
{------------------------------------------------------------------------------}
procedure TGraph.SetColors(NewColors:TGraphColors);
begin
 FColors.Assign(NewColors); //Damit die Komponente die Farbeinstellung beibehlt wenn die Anwendugn gestartet wird
end;

procedure TGraph.SetTransparent(NewValue:Boolean);
begin
 FTransparent := NewValue;
 Repaint;
end;

procedure TGraph.SetGraphFunction(NewFunction:TGraphFunction);
begin
 FGraphFunction := NewFunction;
end;

procedure TGraph.SetName(const NewName: TComponentName);
begin
 inherited;
 Repaint;
end;
{------------------------------------------------------------------------------}
//                            Graph Colors
{------------------------------------------------------------------------------}
procedure TGraphColors.SetBackGround(NewColor:TColor);
begin
 FBackGround:= NewColor;
 if Assigned(@OnChange) then OnChange;
end;

procedure TGraphColors.SetPen(NewColor:TColor);
begin
 FPen := NewColor;
 if Assigned(@OnChange) then OnChange;
end;




procedure Register;
begin
  RegisterComponents('Tutorials', [TGraph]);
end;

end.
 