unit ZeichenRoutinen;
(***** BEGIN LICENSE BLOCK *****
Version: MPL 1.1

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is Zeichenroutinen.pas .

The Initial Developer of the Original Code is
Florian Kberle.
Portions created by the Initial Developer are Copyright (C) 2004
the Initial Developer. All Rights Reserved.

Contributor(s):

***** END LICENSE BLOCK *****)
interface
uses Windows,Graphics;

function DrawEllipse(const Bitmap:TBitmap; Rect:TRect;const Color:TColor;Border:Single):Boolean;
function DrawRoundButton(Bitmap:TBitmap;Rect:TRect;Color:TColor;Border:Single;Down:Boolean):Boolean;

implementation
  {$DEFINE SaveMode} //bergebenes Rechteck berprfen
type
  TBGRTColor=packed record r,g,b,t:Byte end;
  TBGRColor=packed record b,g,r:Byte end;
  PBGRColor=^TBGRColor;
  TPunkt = record X,Y:Single; end;
function ColorToBGRColor(Color:TColor):TBGRColor;
begin
  Integer(Color) := ColorToRGB(Color);
  result.r := TBGRTColor(Color).r;
  result.g := TBGRTColor(Color).g;
  result.b := TBGRTColor(Color).b;
end;
function CheckRectAndBitmap(var Rect:TRect; const Bitmap:TBitmap):Boolean;
var
  TempInt:Integer;
begin
  Result := False;
  if Bitmap = nil then Exit;
  try
    if Rect.Right < Rect.Left then //Werte sind vertauscht.
    begin //Vertauschen
      TempInt := Rect.Right;
      Rect.Right := Rect.Left;
      Rect.Left := TempInt;
    end;
    if Rect.Bottom < Rect.Top then //Werte sind vertauscht.
    begin //Vertauschen
      TempInt := Rect.Bottom;
      Rect.Bottom := Rect.Top;
      Rect.Top := TempInt;
    end;
    if Rect.Top < 0 then Rect.Top := 0;
    if Rect.Left < 0 then Rect.Left := 0;
    if Rect.Right > Bitmap.Width then Rect.Right := Bitmap.Width;
    if Rect.Bottom > Bitmap.Height then Rect.Bottom := Bitmap.Height;
    Result := True;
  except
    Result := False;
  end;
end;
function AsByte(X:Integer):Byte;
begin
  if X < 0 then result := 0
  else
  if X > 255 then result := 255
  else
  Result := X;
end;

function PunktzupiWinkel(X,Y:Single):Single;
begin
  if X = 0 then
   if Y > 0 then  result := 0.5*pi else result:= 1.5*pi
  else {X <> 0}
  begin
   if X > 0 then
   begin
     if Y < 0 then
      result := 2*pi+Arctan(Y/X)
     else
      result := Arctan(Y/X)
   end
   else
    result := Arctan(Y/X) +pi;
  end;
  Result := 2*pi - Result;
end;

function DrawEllipse(const Bitmap:TBitmap;Rect:TRect;const Color:TColor;Border:Single):Boolean;
var
  Farbe:TBGRColor;
  Pixel:PBGRColor;
  X,Y:Integer;
  Punkt:TPunkt;
  Mitte:TPunkt;
  Width,Height:Integer;
  KreisRadius:Single;
  ScaleX,ScaleY:Single;
  Abstand,Winkel:Single;
  Abweichung:Single;
  Faktor:Single;
begin
  Farbe := ColorToBGRColor(Color);
  if not CheckRectAndBitmap(Rect,Bitmap) then Exit;

  Width := Rect.Right - Rect.Left;
  Height :=  Rect.Bottom - Rect.Top;
  Mitte.X := (Rect.Right + Rect.Left)/2;
  Mitte.Y := (Rect.Bottom + Rect.Top)/2;

  if Width < Height then
  begin
    //Size := Width;
    KreisRadius := Width / 2;
    ScaleX := 1;
    ScaleY := Height/Width;
  end
  else
  begin
    //Size := Height;
    KreisRadius := Height / 2;
    ScaleX := Width/Height;
    ScaleY := 1;
  end;
  Border:= Border/2;
  KreisRadius := KreisRadius-Border;

  Bitmap.PixelFormat := pf24bit;
  for Y := Rect.top to Rect.Bottom-1 do
  begin
    Pixel := Bitmap.ScanLine[Y];
    Inc(Pixel,Rect.Left);
    for X := Rect.left to Rect.Right-1 do
    begin
      Punkt.X := (X - Mitte.X)/ScaleX;
      Punkt.Y := (Y - Mitte.Y)/ScaleY;
      Abstand:= Sqrt(Sqr(Punkt.X)+Sqr(Punkt.Y));
      Winkel:= PunktzupiWinkel(Punkt.X,Punkt.Y);

      if (Abstand > KreisRadius - Border) and (Abstand < KreisRadius + Border) then
      begin
        Abweichung:= Abs(KreisRadius-Abstand);
        Faktor := (Border - Abweichung)/Border;
        Faktor := Faktor;
        Pixel.r := AsByte(Round(Pixel.r*(1-Faktor) + Farbe.r*Faktor));
        Pixel.g := AsByte(Round(Pixel.g*(1-Faktor) + Farbe.g*Faktor));
        Pixel.b := AsByte(Round(Pixel.b*(1-Faktor) + Farbe.b*Faktor));
      (*  *)
      end;
      Inc(Pixel);
    end;
  end;
end;

function DrawRoundButton(Bitmap:TBitmap;Rect:TRect;Color:TColor;Border:Single;Down:Boolean):Boolean;
var
  ColorNr: array [-100..100] of TBGRColor;
  Pixel:PBGRColor;
  bgColor:TBGRTColor;
  Ok:Boolean;
  X,Y,I:Integer;
  Winkel,Abstand:Single;
  KreisRadius:Single;
  Faktor:Single;
  Faktor2:Single;
  Index:Integer;
  AbstandzumRand:Single;
  Width,Height,Size:Integer;
  ScaleX, ScaleY:Single;
  Mitte:TPunkt;
  Punkt:TPunkt;
  LichtFaktor:Single;
  LichtWert:Integer;
begin
  ColorNr[0] := ColorToBGRColor(Color);

  if Down then LichtFaktor := 120 else LichtFaktor := 180;
  for I := low(ColorNr) to -1 do
  begin
    LichtWert := Round(LichtFaktor*I/100);
    ColorNr[I].r := AsByte(ColorNr[0].r + LichtWert);
    ColorNr[I].g := AsByte(ColorNr[0].g + LichtWert);
    ColorNr[I].b := AsByte(ColorNr[0].b + LichtWert);
  end;
  LichtFaktor := 120;
  for I := 0 to high(ColorNr) do
  begin
    LichtWert := Round(LichtFaktor*I/100);
    ColorNr[I].r := AsByte(ColorNr[0].r + LichtWert);
    ColorNr[I].g := AsByte(ColorNr[0].g + LichtWert);
    ColorNr[I].b := AsByte(ColorNr[0].b + LichtWert);
  end;
  {$IFDEF SaveMode}
  if not CheckRectAndBitmap(Rect,Bitmap) then exit;
  {$ENDIF}

  Width := Rect.Right - Rect.Left;
  Height :=  Rect.Bottom - Rect.Top;
  Mitte.X := (Rect.Right + Rect.Left)/2;
  Mitte.Y := (Rect.Bottom + Rect.Top)/2;

  if Width < Height then
  begin
    Size := Width;
    ScaleX := 1;
    ScaleY := Height/Size;
  end
  else
  begin
    Size := Height;
    ScaleX := Width/Size;
    ScaleY := 1;
  end;
  KreisRadius := Size / 2;

  try
    Bitmap.PixelFormat := pf24bit;
    for Y := Rect.top to Rect.Bottom-1 do
    begin
      Pixel := Bitmap.ScanLine[Y];
      Inc(Pixel,Rect.Left);
      for X := Rect.left to Rect.Right-1 do
      begin
        Punkt.X := (X - Mitte.X)/ScaleX;
        Punkt.Y := (Y - Mitte.Y)/ScaleY;
        Abstand:= Sqrt(Sqr(Punkt.X)+Sqr(Punkt.Y));
        Winkel:= PunktzupiWinkel(Punkt.X,Punkt.Y);
        if not Down then
         Faktor := Sin(Winkel-pi/4)
        else
         Faktor := Sin(Winkel-5*pi/4);
        if Abstand <= KreisRadius then
        begin
          AbstandzumRand := KreisRadius-Abstand;
          if AbstandzumRand < Border then
          begin
            Faktor2 := sqr(1-AbstandzumRand/Border);
            if Faktor2 < 0 then
            Faktor2 := 0;
            if Faktor2 > 1 then
            Faktor2 := 1;
            Index := round(Faktor*Faktor2*100);
          end
          else Index := 0;
          if AbstandzumRand < 1 then
          begin
            Pixel.r := AsByte(Round(Pixel.r*(1-AbstandzumRand) + ColorNr[Index].r*AbstandzumRand));
            Pixel.g := AsByte(Round(Pixel.g*(1-AbstandzumRand) + ColorNr[Index].g*AbstandzumRand));
            Pixel.b := AsByte(Round(Pixel.b*(1-AbstandzumRand) + ColorNr[Index].b*AbstandzumRand));
          end
          else
          begin
            Pixel.r := ColorNr[Index].r;
            Pixel.g := ColorNr[Index].g;
            Pixel.b := ColorNr[Index].b;
          end;
        end;
        Inc(Pixel);
      end;
    end;
    Result := True;
  except
    Result := False;
  end;
end;
(*
  bgColor.t := 0;
  repeat
    bgColor.r := random(255);
    bgColor.g := random(255);
    bgColor.b := random(255);
    Ok := True;
    for I := low(ColorNr) to high(ColorNr) do
      if  (bgColor.r = ColorNr[I].r)
      and (bgColor.g = ColorNr[I].g)
      and (bgColor.b = ColorNr[I].b)  then
      begin
        Ok := False;
        Break;
      end;
  until Ok;    *)
end.
